import json
import random

from .mixins import ChartObjectMixin


class BarChart(ChartObjectMixin):
    def __init__(self):
        super().__init__()
        self.type_chart = "bar"

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "yAxes": [
                {
                    "display": True,
                    "ticks": {
                        "beginAtZero": self.begin_at_zero,
                        "stepSize": self.step_size,
                    },
                }
            ]
        }

        return options

    def generate_dataset(self):
        return {
            "labels": self.labels,
            "datasets": [
                {
                    "label": self.tooltips
                    if len(self.tooltips) > 0
                    else self.data_label,
                    "backgroundColor": self.get_colors,
                    "data": self.data,
                }
            ],
        }


class HorizontalBarChart(BarChart):
    def __init__(self):
        super().__init__()
        self.type_chart = "horizontalBar"


class PieChart(ChartObjectMixin):
    def __init__(self):
        super().__init__()
        self.type_chart = "pie"
        self.position = "top"  # top,right, bottom, left

    def generate_options(self):
        context = super().generate_options()
        context["legend"].update({"position": self.position})
        return context

    def generate_dataset(self):
        return {
            "labels": self.labels,
            "datasets": [
                {
                    "label": self.data_label if self.data_label else "",
                    "backgroundColor": self.get_colors,
                    "data": self.data,
                }
            ],
        }


class DoughnutChart(PieChart):
    type_chart = "doughnut"


class PolarAreaChart(PieChart):
    type_chart = "polarArea"


class LineChart(ChartObjectMixin):
    def __init__(self):
        super().__init__()
        self.type_chart = "line"

    def create_node(self, label, data, fill=False, color=None):
        """
            this method create special line node, you must pass parameters
            `label` str -> an label individual node, `data`  list -> data render on chart,
            `fill` bool -> default is False, use this to create area chart
            `color` str -> hex color representation (when fill is True)
        """
        color_data = color if color is not None else self._get_color()
        return {
            "data": list(data),
            "label": label,
            "backgroundColor": self._get_rgba_from_hex(color_data),
            "borderColor": color_data,
            "fill": fill,
        }

    def generate_options(self):
        context = super().generate_options()
        context["scales"] = {
            "yAxes": [
                {
                    "display": True,
                    "ticks": {
                        "beginAtZero": self.begin_at_zero,
                        "stepSize": self.step_size,
                    },
                }
            ]
        }
        return context

    def generate_dataset(self):
        return {"labels": self.labels, "datasets": self.data}

    def build_chart(self):
        return json.dumps(
            {
                "type": self.type_chart,
                "data": self.generate_dataset(),
                "options": self.generate_options(),
            },
            ensure_ascii=False,
        )


class GroupChart(ChartObjectMixin):
    def __init__(self):
        super().__init__()
        self.type_chart = "bar"

    def create_node(self, label, data, color=None):
        """
            This method create an special node to group chart:
            `label` -> (str) text to represent individual data
            `data` -> (list) data to render in chart
            `color` -> (str) hex string representation of color, default is None
        """
        color_data = color if color is not None else self._get_color()
        return {"label": label, "backgroundColor": color_data, "data": list(data)}

    def generate_dataset(self):
        dataset = {"labels": self.labels, "datasets": self.data}

    def build_chart(self):
        return json.dumps(
            {
                "type": self.type_chart,
                "data": self.generate_dataset(),
                "options": self.generate_options(),
            },
            ensure_ascii=False,
        )


class RadarChart(ChartObjectMixin):
    def __init__(self):
        super().__init__()
        self.type_chart = "radar"

    def create_node(self, label, data, color=None):
        color_data = color if color is not None else self._get_rgba_from_hex(color)
        return {
            "label": label,
            "fill": True,
            "backgroundColor": color_data,
            "borderColor": color,
            "pointBorderColor": "#fff",
            "pointBackgroundColor": color,
            "data": list(data),
        }

    def generate_dataset(self):
        return {"labels": self.labels, "datasets": self.data}

    def build_chart(self):
        return json.dumps(
            {
                "type": self.type_chart,
                "data": self.generate_dataset(),
                "options": self.generate_options(),
            },
            ensure_ascii=False,
        )



import json
from django.test import TestCase
from charts.objects import BarChart, PieChart, HorizontalBarChart


class TestCaseChart(TestCase):

    def test_should_return_barchart_data(self):
        chart = BarChart()
        chart.title = "Example charts title"

        chart.labels = ["test 1", "test 2", "test 3", "test 4"]
        chart.data = [2, 3, 10, 6]
        chart.data_label = "Test"

        data = chart.build_chart()
        options = chart.generate_options()

        assert data == expected_data(chart, options)

    def test_should_return_horizontal_barchart_data(self):
        chart = HorizontalBarChart()
        chart.title = "Example charts title"

        chart.labels = ["test 1", "test 2", "test 3", "test 4"]
        chart.data = [2, 3, 10, 6]
        chart.data_label = "Test"

        data = chart.build_chart()
        options = chart.generate_options()

        assert data == expected_data(chart, options)

    def test_should_return_piechart_data(self):
        chart = PieChart()
        chart.title = "Example charts title"

        chart.labels = ["test 1", "test 2", "test 3", "test 4"]
        chart.data = [2, 3, 10, 6]
        chart.data_label = "Test"

        data = chart.build_chart()
        options = chart.generate_options()

        assert data == expected_pie_data(chart, options)

def expected_data(chart, options):
    return json.dumps({
        "type": chart.type_chart,
        "data": {
            "labels": [label for label in chart.labels],
            "datasets": [
                {
                    "label": chart.data_label,
                    "backgroundColor": [color for color in chart.get_colors],
                    "data": [value for value in chart.data]
                }
            ]
        },
        "options": {
            "responsive": options["responsive"],
            "maintainAspectRatio": options["maintainAspectRatio"],
            "legend": {
                "display": chart.legend,
            },
            "title": {
                "fontSize": 14,
                "display": True if chart.title else False,
                "text": chart.title if chart.title else ""
            },
            "scales": {
                "yAxes": [
                    {
                        "display": True,
                        "ticks": {
                            "beginAtZero": chart.begin_at_zero,
                            "stepSize": chart.step_size
                        }
                    }
                ]
            }
        }
    }, ensure_ascii=False)

def expected_pie_data(chart, options):
    return json.dumps({
        "type": chart.type_chart,
        "data": {
            "labels": [label for label in chart.labels],
            "datasets": [
                {
                    "label": chart.data_label,
                    "backgroundColor": [color for color in chart.get_colors],
                    "data": [value for value in chart.data]
                }
            ]
        },
        "options": {
            "responsive": options["responsive"],
            "maintainAspectRatio": options["maintainAspectRatio"],
            "legend": {
                "display": chart.legend,
                "position": chart.position,
            },
            "title": {
                "fontSize": 14,
                "display": True if chart.title else False,
                "text": chart.title if chart.title else ""
            },
        }
    }, ensure_ascii=False)



from django import template

register = template.Library()


@register.inclusion_tag("charts_dashboard/chart.html", takes_context=True)
def render_chart(context, values):
    return {
        "chart": values,
        "tooltips": list(*context["tooltips"]),
        "height": context.get("height"),
        "width": context.get("width"),
    }



import json
import random

from .mixins import BaseChartView


class BarChartView(BaseChartView):

    type_chart = "bar"

    def _generate_data(self):
        return {
            "labels": self.generate_labels(),
            "datasets": self._generate_dataset(self.generate_values()),
        }

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "yAxes": [
                {
                    "display": True,
                    "ticks": {
                        "beginAtZero": self.begin_at_zero,
                        "stepSize": self.step_size,
                    },
                }
            ]
        }

        return options

    def _generate_dataset(self, values: list):
        collection = []
        dataset = {
            "label": self.get_tooltips() if self.get_tooltips() else "",
            "backgroundColor": [self._get_color()] * len(values)
            if not self.colors
            else self.colors,
            "data": values,
        }
        collection.append(dataset)

        return collection


class RadarChartView(BaseChartView):

    type_chart = "radar"

    def generate_labels(self):
        return []

    def generate_values(self):
        return []

    def create_node(self, label, data, color=None):
        color_data = color if color is not None else self._get_color()
        return {
            "label": label,
            "fill": True,
            "backgroundColor": self._get_rgba_from_hex(color_data),
            "borderColor": color_data,
            "pointBorderColor": "#fff",
            "pointBackgroundColor": color_data,
            "data": list(data),
        }

    def _generate_data(self):
        return {"labels": self.generate_labels(), "datasets": self.generate_values()}


class LineChartView(BaseChartView):

    type_chart = "line"

    """
        the params accept `label` string, and `data` list of values (numeric)
        fill is False to default
    """

    def create_node(self, label, data, fill=False, color=None):
        color_data = color if color is not None else self._get_color()
        return {
            "data": list(data),
            "label": label,
            "borderColor": color_data,
            "backgroundColor": self._get_rgba_from_hex(color_data),
            "fill": fill,
        }

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "yAxes": [
                {
                    "display": True,
                    "ticks": {
                        "beginAtZero": self.begin_at_zero,
                        "stepSize": self.step_size,
                    },
                }
            ]
        }
        return json.dumps(options, ensure_ascii=False)

    def _get_color(self):
        return "rgba({},{},{},0.4)".format(
            *map(lambda x: random.randint(0, 255), range(5))
        )

    def _generate_data(self):
        return json.dumps(
            {"labels": self.generate_labels(), "datasets": self.generate_values()},
            ensure_ascii=False,
        )


class GroupChartView(BaseChartView):

    type_bar = "bar"

    def create_node(self, data, label):
        return {
            "label": label,
            "backgroundColor": "#{:02x}{:02x}{:02x}".format(
                *map(lambda x: random.randint(0, 255), range(3))
            ),
            "data": list(data),
        }

    def _generate_data(self):
        return json.dumps(
            {"labels": self.generate_labels(), "datasets": self.generate_values()},
            ensure_ascii=False,
        )


class HorizontalBarChartView(BarChartView):
    type_chart = "horizontalBar"


class PolarAreaChartView(BarChartView):
    type_chart = "polarArea"


class PieChartView(BarChartView):
    type_chart = "pie"
    position = "top"  # top,right, bottom, left

    def generate_options(self):
        options = {
            "legend": {"display": self.legend, "position": self.position},
            "title": {
                "display": True if self.title is not None else False,
                "text": self.title,
            },
        }

        return json.dumps(options, ensure_ascii=False)

    def get_legend_text(self):
        return ""

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["legend"] = self.get_legend_text()
        return context


class DoughnutChartView(PieChartView):
    type_chart = "doughnut"



import os


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = "huahsuasklweoi2ru0iewdkldfs"

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'charts',
]

ROOT_URLCONF = 'tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'tests/testdb.sqlite3'),
    }
}


import json
import random
from abc import ABC, abstractmethod


class BaseChartView(ABC):

    type_chart = None
    title = None
    legend = None
    begin_at_zero = False
    aspect_ratio = True
    step_size = 0.5
    width = 100
    height = 100
    colors = []

    def get_tooltips(self) -> list:
        return []

    @abstractmethod
    def generate_values(self):
        pass

    def generate_options(self):
        return {
            "responsive": True,
            "maintainAspectRatio": self.aspect_ratio,
            "legend": {"display": self.legend},
            "title": {"display": True if self.title else False, "text": self.title},
        }

    @abstractmethod
    def generate_labels(self):
        """
            suggestion: insert colors in self.colors array
            ex.: self.colors = ['#ffff','#121212']
        """
        pass

    def _get_color(self):
        return "#{:02x}{:02x}{:02x}".format(
            *map(lambda x: random.randint(0, 255), range(3))
        )

    def _get_rgba_from_hex(self, color_hex, alpha=0.6):
        color = color_hex.lstrip("#")
        rgb = [int(color[i : i + 2], 16) for i in [0, 2, 4]]

        return "rgba({},{},{}, {})".format(*map(lambda x: x, rgb), alpha)

    def get_context_data(self, **kwargs):
        context = {}
        context["chart"] = json.dumps(
            {
                "type": self.type_chart,
                "data": self.generate_values(),
                "options": self.generate_options(),
            },
            ensure_ascii=False,
        )

        context["tooltips"] = (self.get_tooltips() if self.get_tooltips() else " ",)

        if not self.aspect_ratio:
            context["width"] = self.width
            context["height"] = self.height

        return context


"""
    Objects represent chartjs instances
"""


class ChartObjectMixin(ABC):
    def __init__(self):
        # principal data chart
        self.data = []
        self.labels = []
        self.data_label = ""
        # options values
        self.begin_at_zero = True
        self.aspect_ratio = True
        self.step_size = 0.5
        self.title = None
        self.legend = False
        self.type_chart = None
        self._colors = []
        self.tooltips = []

    @abstractmethod
    def generate_dataset(self):
        pass

    def generate_options(self):
        return {
            "responsive": True,
            "maintainAspectRatio": self.aspect_ratio,
            "legend": {"display": self.legend},
            "title": {
                "fontSize": 14,
                "display": True if self.title else False,
                "text": self.title if self.title else "",
            },
        }

    def _generate_colors(self, labels):
        return [
            "#{:02x}{:02x}{:02x}".format(
                *map(lambda x: random.randint(0, 255), range(3))
            )
            for entry in labels
        ]

    def set_colors(self, colors):
        self._colors = colors

    @property
    def get_colors(self):
        return self._colors

    def _get_color(self):
        return "#{:02x}{:02x}{:02x}".format(
            *map(lambda x: random.randint(0, 255), range(3))
        )

    def _get_rgba_from_hex(self, color_hex):
        color = color_hex.lstrip("#")
        rgb = [int(color[i : i + 2], 16) for i in [0, 2, 4]]

        return "rgba({},{},{},0.6)".format(*map(lambda x: x, rgb))

    def build_chart(self):
        self._colors = (
            self._colors if self._colors else self._generate_colors(self.labels)
        )
        return json.dumps(
            {
                "type": self.type_chart,
                "data": self.generate_dataset(),
                "options": self.generate_options(),
            },
            ensure_ascii=False,
        )
